# Python imports

# Lib imports
import gi
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk
from gi.repository import GLib

# Application imports
from .sink_input_list import SinkInputList, AudioSink
from utils.pulsectl import pulsectl



class BaseContainer(Gtk.Box):
    def __init__(self):
        super(BaseContainer, self).__init__()

        self.pulse        = None
        self.pulse_events = None
        self.sink_inputs  = None
        self.pause_sink   = False

        self._setup_styling()
        self._setup_signals()
        self._subscribe_to_events()
        self._load_widgets()

        self.show_all()


    def _setup_styling(self):
        self.set_orientation(Gtk.Orientation.VERTICAL)
        ctx = self.get_style_context()
        ctx.add_class("base-container")

    def _setup_signals(self):
        ...

    def _subscribe_to_events(self):
        ...

    def _load_widgets(self):
        self._load_pulse_data()

        scroll = SinkInputList(self.pulse)
        box    = scroll.get_box()
        label2 = Gtk.Label("Playback")
        label2.set_xalign(0.0)
        for sink_input in self.sink_inputs:
            sink = AudioSink(self.pulse, sink_input)
            ctx  = sink.get_style_context()
            ctx.add_class("sink-input")
            box.add(sink)

        self.add(label2)
        self.add(scroll)
        self._set_pulse_event_listener()

    def _load_pulse_data(self):
        try:
            label1            = Gtk.Label("Output Devices")
            pulse             = pulsectl.Pulse()
            self.pulse_events = pulsectl.Pulse('event-printer')
            si, sink_inputs, modules, clients, sink_list = pulse.server_info(), \
                                                            pulse.sink_input_list(), \
                                                            pulse.module_list(), \
                                                            pulse.client_list(), \
                                                            pulse.sink_list()

            self.add(label1)
            label1.set_xalign(0.0)
            logger.debug(f"\n\nServer Info\n{si}\n\nSink Inputs:")
            for sink in sink_list:
                sink = AudioSink(pulse, sink)
                ctx  = sink.get_style_context()
                ctx.add_class("sink-device")
                self.add(sink)

            self.pulse       = pulse
            self.sink_inputs = sink_inputs
        except Exception as e:
            logger.debug(f"{e}")

    @daemon_threaded
    def _set_pulse_event_listener(self):
        try:
            self.pulse_events.event_mask_set('all')
            self.pulse_events.event_callback_set(self._sub_threaded_event)
            self.pulse_events.event_listen(timeout = 0)
        except Exception as e:
            logger.debug(f"{e}")

    def _sub_threaded_event(self, eve):
        logger.debug(f"Pulse event: {eve}")
        GLib.idle_add(self.handle_event, *(eve,))

    def handle_event(self, eve):
        if eve.facility == "sink_input":
            if eve.t == "change":
                    event_system.emit("handle_cng_sync_input", (eve.index,))

            if eve.t == "new":
                    event_system.emit("handle_new_sync_input", (eve.index,))

            if eve.t == "remove":
                    event_system.emit("handle_del_sync_input", (eve.index,))
